/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */

#include "config.h"
#include <stdio.h>

#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/Frame.h>
#include <Xm/BulletinB.h>
#include "xmap.h"
#include "mode.h"
#include "debug.h"

extern Widget draw_w;
extern Widget toplevel;

static Widget dialog = NULL;
static Widget form;
static Widget bul;
static Widget ok;
static Widget cancel;
static Widget help;
static Widget latitude, longitude, zone;
static Widget l1, l2, l3;

static double loc_longitude;
static double loc_latitude;


static int answer = ANS_NONE;

static void ok_cb(Widget dialog, XtPointer client_data, XtPointer call_data)
{
    answer = ANS_OK;
}

static void cancel_cb(Widget dialog, XtPointer client_data, XtPointer call_data)
{
    answer = ANS_CANCEL;
//    if (mode_is(CALIBRATE))
//	unset_cal_toggle();
}

int lldialog(char *title, double lat, double lon)
{
    Arg args[100];
    int n;
    Pixel foreground, background;
    XtAppContext app = XtWidgetToApplicationContext(toplevel);
    XmString s;
    char bufp[256];
    char bufp1[21];
    char bufp2[21];

    if (!dialog) {
	foreground = get_pixel(draw_w, "Black");
	background = get_pixel(draw_w, "Snow");
	dialog = XtVaCreatePopupShell("latlon",
			   xmDialogShellWidgetClass, GetTopShell(draw_w),
				      XmNdeleteResponse, XmUNMAP,
				      XmNresizePolicy, XmRESIZE_NONE,
				      XmNnoResize, True,
				      XmNwidth, 290,
				      XmNminWidth, 290,
				      XmNmaxWidth, 290,
				      XmNheight, 130,
				      XmNminHeight, 130,
				      XmNmaxHeight, 130,
				      NULL);

	bul = XtVaCreateWidget("bul",
			       xmBulletinBoardWidgetClass, dialog,
			       XmNforeground, foreground,
			       XmNautoUnmanage, False,
			       XmNdialogStyle, XmDIALOG_WORK_AREA,
			       NULL);
	XtAddCallback(bul, XmNunmapCallback, (void *) cancel_cb, NULL);

	form = XtVaCreateWidget("l0", xmLabelWidgetClass, bul,
				XmNy, 7,
				NULL);

	l1 = XtVaCreateManagedWidget("l1", xmLabelWidgetClass, bul,
				     XmNy, 35,
				     NULL);

	l2 = XtVaCreateManagedWidget("l2", xmLabelWidgetClass, bul,
				     XmNy, 70,
				     NULL);

	s = XmStringCreateSimple("Zone:");
	l3 = XtVaCreateWidget("l3", xmLabelWidgetClass, bul,
				     XmNlabelString, s,
				     XmNy, 105,
				     NULL);
	XmStringFree(s);

	latitude = XtVaCreateManagedWidget("t1", xmTextFieldWidgetClass, bul,
					   XmNhighlightThickness, 0,
					   XmNshadowThickness, 1,
					   XmNmarginWidth, 2,
					   XmNmarginHeight, 2,
					   XmNbackground, background,
					   XmNx, 80,
					   XmNy, 33,
					   XmNwidth, 100,
					   NULL);

	longitude = XtVaCreateManagedWidget("t2", xmTextFieldWidgetClass, bul,
					    XmNhighlightThickness, 0,
					    XmNshadowThickness, 1,
					    XmNmarginWidth, 2,
					    XmNmarginHeight, 2,
					    XmNbackground, background,
					    XmNx, 80,
					    XmNy, 68,
					    XmNwidth, 100,
					    NULL);

	zone = XtVaCreateWidget("t3", xmTextFieldWidgetClass, bul,
				XmNhighlightThickness, 0,
				XmNshadowThickness, 1,
				XmNmarginWidth, 2,
				XmNmarginHeight, 2,
				XmNbackground, background,
				XmNx, 80,
				XmNy, 103,
				XmNwidth, 100,
				NULL);

	cancel = XtVaCreateManagedWidget("Cancel",
					 xmPushButtonWidgetClass, bul,
					 XmNforeground, foreground,
					 XmNhighlightThickness, 0,
					 XmNwidth, 60,
					 XmNx, 200,
					 XmNy, 68,
					 NULL);
	XtAddCallback(cancel, XmNactivateCallback, cancel_cb, NULL);

	ok = XtVaCreateManagedWidget("OK",
				     xmPushButtonWidgetClass, bul,
				     XmNforeground, foreground,
				     XmNhighlightThickness, 0,
				     XmNwidth, 60,
				     XmNx, 200,
				     XmNy, 33,
				     NULL);
	XtAddCallback(ok, XmNactivateCallback, ok_cb, NULL);

	help = XtVaCreateManagedWidget("Help",
				       xmPushButtonWidgetClass, bul,
				       XmNforeground, foreground,
				       XmNhighlightThickness, 0,
				       XmNwidth, 60,
				       XmNx, 200,
				       XmNy, 103,
				       NULL);
	XtAddCallback(help, XmNactivateCallback, show_help_cb, "loc_ll.html");
    }
    snprintf(bufp, 255, "Enter as %s (%d)", get_sform(options.format), options.format);
    s = XmStringCreateSimple(bufp);
    XtVaSetValues(form,
		  XmNlabelString, s,
		  NULL);
    XmStringFree(s);

    if (options.format == UTM) {
	XtManageChild(zone);
	XtManageChild(l3);
	s = XmStringCreateSimple("Easting:");
	XtVaSetValues(l1, XmNlabelString, s, NULL);
	XmStringFree(s);

	s = XmStringCreateSimple("Northing:");
	XtVaSetValues(l2, XmNlabelString, s, NULL);
	XmStringFree(s);
	if (lat != 0.0 || lon != 0.0) {
		char azone[8];
		double x, y;

		DegToUTM(lat, lon, azone, &x, &y);
		snprintf(bufp, 255, "%.0f", x);
		XmTextFieldSetString(latitude, bufp);
		snprintf(bufp, 255, "%.0f", y);
		XmTextFieldSetString(longitude, bufp);
		XmTextFieldSetString(zone, azone);
	}
    } else {
	XtUnmanageChild(zone);
	XtUnmanageChild(l3);
	s = XmStringCreateSimple("Latitude:");
	XtVaSetValues(l1, XmNlabelString, s, NULL);
	XmStringFree(s);

	s = XmStringCreateSimple("Longitude:");
	XtVaSetValues(l2, XmNlabelString, s, NULL);
	XmStringFree(s);
	if (lat != 0.0 || lon != 0.0) {
	    toUserFormatLon(options.format, lon, bufp2, 21);
	    toUserFormatLon(options.format, lat, bufp1, 21);
	    XmTextFieldSetString(longitude, bufp2);
	    XmTextFieldSetString(latitude, bufp1);
	}
    }

    XtManageChild(bul);
    XtManageChild(form);

    XtVaSetValues(dialog,
		  XmNtitle, title,
		  NULL);
    XtManageChild(dialog);
    XtPopup(dialog, XtGrabNone);

    answer = ANS_NONE;

    while (answer == ANS_NONE)
	XtAppProcessEvent(app, XtIMAll);

    XtPopdown(dialog);

    if (answer == ANS_OK) {
	if (options.format == UTM) {
		 aUTMtoDeg(XmTextFieldGetString(zone),
				XmTextFieldGetString(latitude),
				XmTextFieldGetString(longitude),
				&loc_latitude, &loc_longitude);
	}
	else {
		loc_latitude = fromUserFormat(options.format,
				      XmTextFieldGetString(latitude));
		loc_longitude = fromUserFormat(options.format,
				       XmTextFieldGetString(longitude));
	}
//fprintf(stderr,"lat: %f lon: %f\n",loc_latitude,loc_longitude);
    }
    /* make sure the dialog goes away before returning. Sync with server
     * and update the display.
     */
    XSync(XtDisplay(dialog), 0);
    XmUpdateDisplay(toplevel);

    return (answer);
}

double getlat(void)
{
    return loc_latitude;
}

double getlon(void)
{
    return loc_longitude;
}


static double mlat, mlon;
static int mvalid = 0;

void getlatlon()
{
    extern Dimension width, height;
    Position x, y;

    if (options.format == UTM)
	lldialog("Locate by UTM", 0.0, 0.0);
    else
	lldialog("Locate by Latitude/Longitude", 0.0, 0.0);

//fprintf(stderr,"lat %f   lon %f\n",loc_latitude, loc_longitude);
    if (answer==ANS_OK) {
	mlat = loc_latitude;
	mlon = loc_longitude;
	mvalid = 1;
	
	latlontoxy(loc_latitude, loc_longitude, &x, &y);
	center(x, y);
	latlontoxy(loc_latitude, loc_longitude, &x, &y);
	if ( mapdata.type != VECTOR && (x<0 || y<0 || x>=width || y>=height))
	    error("This position is not on\nthe current map!\n", 0);
	else mark_location(x, y);
    }
}

void redraw_location(void)
{
    extern Dimension width, height;
    Position x, y;

    if (mvalid) {
	latlontoxy(mlat, mlon, &x, &y);
	if ( mapdata.type != VECTOR && (x<0 || y<0 || x>=width || y>=height))
	    return;
	else mark_location(x, y);
    }
}
