/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */

#include "config.h"
#include <stdio.h>
#include <Xm/RowColumn.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include "xmap.h"
#include "gps.h"
#include "debug.h"

extern XtAppContext app;
extern Widget toplevel;

extern Widget draw_w;
static Widget dialog;
static void dlg_callback();
static void alarm_callback();

static int answer;

static void create_dialog()
{
    Arg args[5];
    int n = 0;

    XtSetArg(args[n], XmNautoUnmanage, False); n++;
    XtSetArg(args[n], XmNhighlightThickness, 0); n++;
    XtSetArg(args[n], XmNtitle, "Warning"); n++;
    dialog = XmCreateWarningDialog(draw_w, "warning", args, n);
    XtAddCallback(dialog, XmNcancelCallback, dlg_callback, (void *)1);
    XtAddCallback(dialog, XmNokCallback, dlg_callback, (void *)2);
    XtAddCallback(dialog, XmNhelpCallback, (XtCallbackProc) show_help_cb, "r_exist.html");
}

int warning(char *errmsg)
{
    XmString t;
    char bufp[1024];

    if (!dialog)
	create_dialog();

    snprintf(bufp, 1023, "%s", errmsg);

    t = XmStringCreateLtoR(bufp, XmSTRING_DEFAULT_CHARSET);
    XtVaSetValues(dialog,
		  XmNmessageString, t,
		  XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		  NULL);
    XmStringFree(t);

    XtManageChild(dialog);
    XtPopup(XtParent(dialog), XtGrabNone);

    answer = ANS_NONE;

    while (answer == ANS_NONE)
        XtAppProcessEvent(app, XtIMAll);

    XtPopdown(XtParent(dialog));

    /* make sure the dialog goes away before returning. Sync with server
     * and update the display.
     */
    XSync(XtDisplay(dialog), 0);
    XmUpdateDisplay(toplevel);

    return (answer - 1);
}

void prox_alarm(char *msg)
{
    XmString t;
    char bufp[1024];
    static Widget dialog = NULL;

    if (!dialog) {
	Arg args[5];
	int n = 0;

	XtSetArg(args[n], XmNautoUnmanage, False); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNtitle, "Warning"); n++;
	dialog = XmCreateWarningDialog(draw_w, "warning", args, n);
	XtAddCallback(dialog, XmNokCallback, alarm_callback, dialog);
    }
    snprintf(bufp, 1023, "%s", msg);

    t = XmStringCreateLtoR(bufp, XmSTRING_DEFAULT_CHARSET);
    XtVaSetValues(dialog,
		  XmNmessageString, t,
		  XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		  NULL);
    XmStringFree(t);

    XtManageChild(dialog);
    XtPopup(XtParent(dialog), XtGrabNone);
}


static void dlg_callback(Widget dialog, XtPointer client_data, XtPointer call_data)
{
	answer = (int)client_data;
}

static void alarm_callback(Widget dialog, XtPointer client_data, XtPointer call_data)
{
    XtPopdown(XtParent((Widget)client_data));
    navigate_next();
}
