/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */
#include "config.h"

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <time.h>
#include <errno.h>

#include <X11/Xos.h>
#include <Xm/DialogS.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/FileSB.h>
#include <Xm/MainW.h>
#include <Xm/RowColumn.h>
#include <Xm/LabelG.h>
#include <Xm/Form.h>
#include <Xm/List.h>
#include <Xm/ScrollBar.h>
#include <Xm/PushB.h>
#include <X11/xpm.h>
#include <X11/cursorfont.h>

#include "xmap.h"
#include "mode.h"
#include "gps.h"
#include "Xbae/Matrix.h"
#include "debug.h"

#define CH_NOTHING	0
#define CH_COORD	1
#define CH_OTHER	2

extern struct gpsdata gpsdata;

extern Widget draww;
extern void *do_route();
extern void *start_route();
extern void *wp_add();
extern void *d_route_p();
extern void show_help_cb();

static void *croute;
static char *cfn;
static char *header;

static Pixel bg[12];

static Widget dialog, routedialog, mw, start, help, cancel;

void routes_cb()
{
    if (!mode_is(CROUTE)) {
	if (!route_flush(True))
	    return;
    }
    select_iroute_file(draww);
}

void routen_cb()
{
    void *r;

    if (!mode_is(CROUTE)) {
	if (!route_flush(True))
	    return;
    }
    r = do_route();		// in draw.c

    if (r)
	croute = r;
}

void routex_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
    if (mode_is(CROUTE))
	do_route();		// Quit route enrty mode

    if (croute)
	navigate(w, client_data, call_data);
    else
	error("You have to select a route first!\n", 0);
}


void routec_cb()
{
    if (mode_is(CROUTE))
	do_route();		// Quit route enrty mode

    route_flush(False);
}

void routev_cb()
{
    if (!croute)
	return;
    if (mode_is(CROUTE))
	do_route();		// Quit route enrty mode

    select_route_file(draww, cfn);
}

int route_save(char *fn)
{
    FILE *fp;
    time_t thistime;
    int i, n;
    WP *wp;
    char *name, *description, *action;

    if ((fp = fopen(fn, "w")) == NULL) {
	error("Can't save route!\n", errno);
	return;
    }
    n = d_route_nelements(croute);

    time(&thistime);
    if (!header)
	fprintf(fp, "hugoroute 1.0 %s", asctime(gmtime(&thistime)));
    else
	fprintf(fp, "%s", header);

    for (i = 0; i < n; i++) {
	wp = d_route_p(i);	// get the corresponding WP

	name = wp->name ? wp->name : "";
	action = wp->action ? wp->action : "";
	description = wp->description ? wp->description : "";
	fprintf(fp, "%s,%lf,%lf,%lf,%s,%s\n",
		name, wp->lat, wp->lon, wp->alarm,
		description, action);
    }
    fclose(fp);
    is_now_saved(croute);
}

void route_read(char *fn)
{
    char bufp[128];
    WP *wp;
    char *p, *r;
    double lat, lon, alarm;
    FILE *fp;

    if ((fp = fopen(fn, "r")) == NULL) {
	error("Can't read route!\n", errno);
	return;
    }
    if (!fgets(bufp, 128, fp)) {
	error("Can't read header from file!\n", errno);
	fclose(fp);
	return;
    }
    if (strncmp(bufp, "hugoroute 1.0 ", 14)) {
	error("Not a route file, or incompatible version!\n", 0);
	fclose(fp);
	return;
    }

    if (header) free(header);
    header = strdup(bufp);

    croute = start_route();	// in draw.c

    while (fgets(bufp, 128, fp)) {
	p = bufp;

	wp = wp_add();

	r = mystrsep2(&p, ",\n");
	if (r)
	    wp->name = strdup(r);

	r = mystrsep2(&p, ",\n");
	sscanf(r, "%lf", &(wp->lat));

	r = mystrsep2(&p, ",\n");
	sscanf(r, "%lf", &(wp->lon));

	r = mystrsep2(&p, ",\n");
	sscanf(r, "%lf", &(wp->alarm));

	r = mystrsep2(&p, ",\n");
	if (r)
	    wp->description = strdup(r);

	r = mystrsep2(&p, ",\n");
	if (r)
	    wp->action = strdup(r);
    }

    refresh(False);
    fclose(fp);
}

void forget_route()
{
    croute = NULL;
    cfn = NULL;
    if (header)
	free(header);
    header = NULL;
}

int route_flush(int save)
{
    int i;

    if (croute) {
	if (is_modified(croute)) {
	    if (save) {
		i = warning("The current route has not been saved!\n"
			    "Do you want to save it now?");
		if (!i)
		    return FALSE;
		if (!select_route_file(draww, cfn))
		    return FALSE;
	    } else {
		i = warning("The current route has not been saved!\n"
		            "Do you want to clear it anyway?");
		if (!i)
		    return FALSE;
	    }
	}
	route_delete(croute);
	if (routedialog) XtPopdown(routedialog);
	stop_navigate();
	forget_route();
    }
    return TRUE;
}

static void cancel_cb(Widget w, XtPointer client_data, XmAnyCallbackStruct * cbs)
{
    XtPopdown(routedialog);
}

static void ok_cb(Widget w, XtPointer client_data, XmAnyCallbackStruct * cbs)
{
    XtPopdown(routedialog);
}

static void WriteCB(Widget w, XtPointer cd, XbaeMatrixWriteCellCallbackStruct * cbs)
{
    int changed = CH_NOTHING;
    double temp;
    WP *wp;

    if (cbs->row < d_route_nelements(croute)) {
	wp = d_route_p(cbs->row);	// get the corresponding WP

	switch (cbs->column) {
	case 1:
	    if (wp->name)
		free(wp->name);
	    wp->name = strdup(cbs->string);
	    changed |= CH_OTHER;
	    break;

	case 2:			// Latitude
	    temp = fromUserFormat(options.format, cbs->string);
	    if (temp != wp->lat) {
		wp->lat = temp;
		changed = CH_COORD;
	    }
	    break;
 

	case 3:			// Longitude
	    temp = fromUserFormat(options.format, cbs->string);
	    if (temp != wp->lon) {
		wp->lon = temp;
		changed = CH_COORD;
	    }
	    break;

	case 9:
	    wp->alarm = 150 * 100;
	    changed |= CH_OTHER;
	    break;

	case 10:
	    if (wp->description)
		free(wp->description);
	    wp->description = strdup(cbs->string);
	    changed |= CH_OTHER;
	    break;

	case 11:
	    if (wp->action)
		free(wp->action);
	    wp->action = strdup(cbs->string);
	    changed |= CH_OTHER;
	    break;
	}
	if (changed) {
	    if (routedialog) XbaeMatrixRefreshCell(w, cbs->row, cbs->column);
	    XbaeMatrixCancelEdit(w, True);
	    if (changed && CH_COORD) {
		if (routedialog) XbaeMatrixRefresh(mw);	// recalculate distances & ...
		refresh(0);			// redraw the route
	    }
	    is_now_changed(croute);
	}
    }
}

static void EnterCB(Widget w, XtPointer cd, XbaeMatrixEnterCellCallbackStruct * cbs)
{
    if (cbs->row < d_route_nelements(croute)) {
	switch (cbs->column) {
	case 1:			// Name
	case 2:			// Latitude
        case 3:			// Longitude
	case 9:			// Alarm
	case 10:		// Description
	case 11:		// Action
	    cbs->doit = True;
	    return;
	}
    }
    cbs->doit = False;
}

static void TraverseCB(Widget w, XtPointer cd, XbaeMatrixTraverseCellCallbackStruct * cbs)
{
    if (cbs->next_row >= d_route_nelements(croute))
	cbs->next_row = 0;

    if (cbs->next_column <= 3)		// 1, 2, and 3 are editable
	return;
    if (cbs->next_column < 9) {		// 9 to ... are editable
	cbs->next_column = 9;
	return;
    }
}


DrawCB(Widget w, XtPointer cd, XbaeMatrixDrawCellCallbackStruct * cbs)
{ 
    static char bufp[128];
    double d0, d1;
    char *p;
    WP *wc, *wo;

    if (cbs->row < d_route_nelements(croute)) {
	cbs->type = XbaeString;
	if (cbs->row)
	    wo = d_route_p(cbs->row - 1);	// get the previous WP

	wc = d_route_p(cbs->row);	// get the corresponding WP

	switch (cbs->column) {
	case 0:		// Nr.
	    snprintf(bufp, 255, "%d", cbs->row + 1);
	    cbs->string = bufp;
	    break;

	case 1:		// Name
	    cbs->string = wc->name ? wc->name : "";
	    break;

	case 2:		// Latitude
	    toUserFormatLat(options.format, wc->lat, bufp, 255);
	    cbs->string = bufp;
	    break;

	case 3:		// Longitude
	    toUserFormatLon(options.format, wc->lon, bufp, 255);
	    cbs->string = bufp;
	    break;

	case 4:		// Distance
	    bufp[0] = '\0';
	    switch (unit) {
	    case METRIC:
		p = "Km";
		break;
	    case NAUTICAL:
		p = "nm";
		break;
	    case ENGLISH:
		p = "mi";
		break;
	    }

	    if (cbs->row) {
		d0 = cv_dist(distance(wo->lat, wo->lon, wc->lat, wc->lon));
		if (d0>=1000) snprintf(bufp, 255, "%.0lf%s", d0, p);
		else snprintf(bufp, 255, "%.1lf%s", d0, p);
	    } else if (gps_valid()) {
		d0 = cv_dist( 
		    distance(gpsdata.latitude, gpsdata.longitude, wc->lat, wc->lon));
		if (d0>=1000) snprintf(bufp, 255, "%.0lf%s", d0, p);
		else snprintf(bufp, 255, "%.1lf%s", d0, p);
	    }
	    cbs->string = bufp;
	    break;

	case 5:		// Bearing
	    bufp[0] = '\0';

	    if (cbs->row) {
		snprintf(bufp, 255, "%03.0lf", heading(wo->lat, wo->lon, wc->lat, wc->lon));
	    } else if (gps_valid()) {
		snprintf(bufp, 255, "%03.0lf",
			heading(gpsdata.latitude, gpsdata.longitude, wc->lat, wc->lon));
	    }
	    cbs->string = bufp;
	    break;

	case 6:		// Easting
	    DegToUTM(wc->lat, wc->lon, bufp, &d0, &d1);
	    snprintf(bufp, 255, "%.0lf", d0);
	    cbs->string = bufp;
	    break;

	case 7:		// Northing
	    DegToUTM(wc->lat, wc->lon, bufp, &d0, &d1);
	    snprintf(bufp, 255, "%.0lf", d1);
	    cbs->string = bufp;
	    break;

	case 8:		// Zone
	    DegToUTM(wc->lat, wc->lon, bufp, &d0, &d1);
	    cbs->string = bufp;
	    break;

	case 9:		// Alarm Radius
	    cbs->string = "150m";
	    break;

	case 10:		// Description
	    cbs->string = wc->description ? wc->description : "";
	    break;

	case 11:		// Action
	    cbs->string = wc->action ? wc->action : "Default";
	    break;
	}
    }
}


void routee_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
    Pixel foreground;
    int n;

    if (mode_is(CROUTE))
	do_route();		// Quit route enrty mode

    if (!croute) {
	beep();
	return;
    }
    if (!routedialog) {

	bg[ 0] = bg[ 1] = bg[ 2] = bg[ 3] =
	bg[ 4] = bg[ 5] = bg[ 6] = bg[ 7] = 
	bg[ 8] = bg[ 9] = bg[10] = bg[11] = get_pixel(w, "Grey");

	bg[ 1] = bg[ 2] = bg[ 3] = get_pixel(w, "White");
	bg[ 9] = bg[11] = get_pixel(w, "Red");
	bg[10] = get_pixel(w, "White");

	foreground = get_pixel(w, "Black");
	routedialog = XtVaCreatePopupShell("routedialog",
				xmDialogShellWidgetClass, GetTopShell(w),
					   XmNtitle, "Route Editor",
					   XmNdeleteResponse, XmUNMAP,
					   XmNresizePolicy, XmRESIZE_NONE,
					   XmNnoResize, True,
					   XmNmarginWidth, 2,
					   XmNmarginHeight, 2,
					   NULL);

	dialog = XtVaCreateWidget("dialog",
				  xmBulletinBoardWidgetClass, routedialog,
				  XmNforeground, foreground,
				  XmNautoUnmanage, False,
				  XmNdialogStyle, XmDIALOG_WORK_AREA,
				  NULL);
	XtAddCallback(dialog, XmNunmapCallback, (void *) cancel_cb, NULL);

	// Most of the magic is done with resources: resource.c
	mw = XtVaCreateManagedWidget("mw", xbaeMatrixWidgetClass, dialog,
				     NULL);
	XtAddCallback(mw, XmNdrawCellCallback, (XtCallbackProc)DrawCB, NULL);
	XtAddCallback(mw, XmNwriteCellCallback, (XtCallbackProc)WriteCB, NULL);
	XtAddCallback(mw, XmNenterCellCallback, (XtCallbackProc)EnterCB, NULL);
	XtAddCallback(mw, XmNtraverseCellCallback, (XtCallbackProc)TraverseCB, NULL);

	cancel = XtVaCreateManagedWidget("Cancel",
					 xmPushButtonWidgetClass, dialog,
					 XmNforeground, foreground,
					 XmNhighlightThickness, 1,
					 XmNhighlightColor, foreground,
					 XmNwidth, 60,
					 XmNx, 140,
					 XmNy, 246,
					 NULL);
	XtAddCallback(cancel, XmNactivateCallback, (XtCallbackProc) cancel_cb, NULL);

	start = XtVaCreateManagedWidget("OK",
					xmPushButtonWidgetClass, dialog,
					XmNforeground, foreground,
					XmNhighlightThickness, 1,
					XmNhighlightColor, foreground,
					XmNwidth, 60,
					XmNx, 40,
					XmNy, 246,
					NULL);
	XtAddCallback(start, XmNactivateCallback, (XtCallbackProc) ok_cb, NULL);

	help = XtVaCreateManagedWidget("Help",
				       xmPushButtonWidgetClass, dialog,
				       XmNforeground, foreground,
				       XmNhighlightThickness, 1,
				       XmNhighlightColor, foreground,
				       XmNwidth, 60,
				       XmNx, 240,
				       XmNy, 246,
				       NULL);
	XtAddCallback(help, XmNactivateCallback, (XtCallbackProc) show_help_cb, "routeedit.html");

	XtManageChild(mw);
    }
    XtVaGetValues(mw, XmNrows, &n, NULL);
    XbaeMatrixDeleteRows(mw, 0, n);

    n = d_route_nelements(croute);
    n = n < 8 ? 8 : n;
    XbaeMatrixAddRows(mw, 0, NULL, NULL, NULL, n);
    XbaeMatrixSetColumnBackgrounds(mw, 0, bg, 12);

    XtManageChild(dialog);
    XtManageChild(routedialog);
    XtPopup(routedialog, XtGrabNone);

}


int route_test(int x, int y, int *wp)
{
    int n, i, ret = 0;
    WP *wp1, *wp2;
    Position x1, y1, x2, y2;

    if (croute != NULL) {
	n = d_route_nelements(croute);

	wp1 = d_route_p(0);

	for (i = 1; i < n; i++) {
	    wp2 = d_route_p(i);	// get the corresponding WP

	    latlontoxy(wp1->lat, wp1->lon, &x1, &y1);
	    latlontoxy(wp2->lat, wp2->lon, &x2, &y2);

	    if (IsPnt(x1, y1, x, y, 3))
		ret = 1;
	    else if (IsPnt(x2, y2, x, y, 3))
		ret = 3;

	    else if (PntOnLine(x1 >> 2, y1 >> 2, x2 >> 2, y2 >> 2, x >> 2, y >> 2) == 2)
		ret = 2;

	    wp1 = wp2;
	    if (ret)
		break;
	}
	if (ret==1) --i;
	*wp = i;
    }
    return ret;
}

int route_change(int sx, int sy, int ex, int ey)
{
    int i;
    WP *wp;

    if (croute) {
	undraw_route(croute);
	switch (route_test(sx, sy, &i)) {
	case 1:
	case 3:
	    wp = d_route_p(i);	// get the corresponding WP

	    wp->lat = y2lat(ey);
	    wp->lon = x2lon(ex);
	    xytolatlon(ex, ey, &(wp->lat), &(wp->lon));
	    is_now_changed(croute);
	    if (routedialog) XbaeMatrixRefresh(mw);
	    break;

	case 2:
	    beep();
	    break;
	}
    }
    return 0;
}
